@@ -31,6 +31,7 @@ struct _TangleViewPrivate {
 	gchar* filename;
 	ClutterScript* script;
 	guint script_id;
+	guint loaded : 1;
 };
 
 static void add_orphans_to_container(TangleView* view);
@@ -50,27 +51,19 @@ ClutterActor* tangle_view_new_with_layout(TangleLayout* layout) {
 	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_VIEW, "layout", layout, NULL));
 }
 
-void tangle_view_load_from_file(TangleView* view, const gchar* filename) {
+void tangle_view_set_filename(TangleView* view, const gchar* filename) {
 	GError* error = NULL;
 
-	if (!view->priv->script) {
-		view->priv->script = clutter_script_new();
-	}
-	if (view->priv->script_id) {
-		/* TODO: unparent every children. */
-		clutter_script_unmerge_objects(view->priv->script, view->priv->script_id);
-		view->priv->script_id = 0;
-	}
-	g_free(view->priv->filename);
-	view->priv->filename = g_strdup(filename);
-	
-	if (!(view->priv->script_id =
-	      clutter_script_load_from_file(view->priv->script, filename, &error))) {
-		g_critical("Cannot load JSON file '%s': %s\n", filename, error->message);
-		g_error_free(error);
-	} else {
-		add_orphans_to_container(view);
-		clutter_script_connect_signals(view->priv->script, view);
+	if (g_strcmp0(view->priv->filename, filename)) {
+		if (view->priv->script && view->priv->script_id) {
+			/* TODO: unparent every children. */
+			clutter_script_unmerge_objects(view->priv->script, view->priv->script_id);
+			view->priv->script_id = 0;
+		}
+		g_free(view->priv->filename);
+		view->priv->filename = g_strdup(filename);
+		view->priv->loaded = FALSE;
+		g_object_notify(G_OBJECT(view), "filename");
 	}
 }
 
@@ -84,6 +77,29 @@ ClutterScript* tangle_view_get_script(TangleView* view) {
 	return view->priv->script;
 }
 
+void tangle_view_load_now(TangleView* view) {
+	GError* error = NULL;
+
+	g_return_if_fail(view->priv->filename != NULL);
+
+	if (!view->priv->loaded) {
+		if (!view->priv->script) {
+			view->priv->script = clutter_script_new();
+		}
+
+		if (!(view->priv->script_id =
+		      clutter_script_load_from_file(view->priv->script, view->priv->filename, &error))) {
+			g_critical("Cannot load JSON file '%s': %s\n", view->priv->filename, error->message);
+			g_error_free(error);
+		} else {
+			add_orphans_to_container(view);
+			clutter_script_connect_signals(view->priv->script, view);
+		}
+
+		view->priv->loaded = TRUE;
+	}
+}
+
 void tangle_view_start_timelines(TangleView* view) {
 	/* TODO: Implement! */
 }
@@ -132,6 +148,18 @@ static ClutterAnimation* tangle_view_animate_transition(TangleActor* actor, Clut
 	return animation;
 }
 
+static void tangle_view_map(ClutterActor* actor) {
+	TangleView* view;
+	
+	view = TANGLE_VIEW(actor);
+	
+	if (view->priv->filename && !view->priv->loaded) {
+		tangle_view_load_now(view);
+	}
+
+	CLUTTER_ACTOR_CLASS(tangle_view_parent_class)->map(actor);	
+}
+
 static void tangle_view_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
 	TangleView* view;
 	
@@ -139,7 +167,7 @@ static void tangle_view_set_property(GObject* object, guint prop_id, const GValu
 
 	switch (prop_id) {
 		case PROP_FILENAME:
-			tangle_view_load_from_file(view, g_value_get_string(value));
+			tangle_view_set_filename(view, g_value_get_string(value));
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -185,12 +213,15 @@ static void tangle_view_dispose(GObject* object) {
 
 static void tangle_view_class_init(TangleViewClass* view_class) {
 	GObjectClass* gobject_class = G_OBJECT_CLASS(view_class);
+	ClutterActorClass* clutter_actor_class = CLUTTER_ACTOR_CLASS(view_class);
 	TangleActorClass* actor_class = TANGLE_ACTOR_CLASS(view_class);
 
 	gobject_class->finalize = tangle_view_finalize;
 	gobject_class->dispose = tangle_view_dispose;
 	gobject_class->set_property = tangle_view_set_property;
 	gobject_class->get_property = tangle_view_get_property;
+	
+	clutter_actor_class->map = tangle_view_map;
 
 	actor_class->animate_transition = tangle_view_animate_transition;
 
@@ -199,7 +230,7 @@ static void tangle_view_class_init(TangleViewClass* view_class) {
 	 */
 	g_object_class_install_property(gobject_class, PROP_FILENAME,
 	                                g_param_spec_string("filename",
-	                                "File name",
+	                                "Filename",
 	                                "The name of the JSON file containing child objects",
 	                                NULL,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
@@ -247,8 +278,7 @@ static void add_orphans_to_container(TangleView* view) {
 	for (list = objects; list; list = list->next) {
 		if (list->data != (gpointer)view && CLUTTER_IS_ACTOR(list->data) && !TANGLE_IS_VIEW(list->data)) {
 			actor = CLUTTER_ACTOR(list->data);
-			if (!clutter_actor_get_parent(actor)) {
-g_print("parent %p %s %s\n", actor, clutter_actor_get_name(actor), clutter_get_script_id(G_OBJECT(actor)));
+			if (!clutter_actor_get_parent(actor) && !tangle_actor_get_warehouse_parent(actor)) {
 				clutter_container_add_actor(CLUTTER_CONTAINER(view), actor);
 			}
 		}
