@@ -36,7 +36,8 @@ enum {
 	PROP_BACKGROUND_COLOR,
 	PROP_BACKGROUND_COLOR_SET,
 	PROP_BACKGROUND_ACTOR,
-	PROP_PREFER_BACKGROUND_SIZE
+	PROP_PREFER_BACKGROUND_SIZE,
+	PROP_PREFER_BACKGROUND_SIZE_SET
 };
 
 struct _TangleWidgetPrivate {
@@ -52,6 +53,7 @@ struct _TangleWidgetPrivate {
 	guint padding_set : 1;
 	guint background_color_set : 1;
 	guint prefer_background_size : 1;
+	guint prefer_background_size_set : 1;
 };
 
 static void tangle_widget_add_impl(TangleWidget* container, ClutterActor* actor, ClutterActor* existing, gboolean before);
@@ -263,13 +265,27 @@ gboolean tangle_widget_get_prefer_background_size(TangleWidget* widget) {
 }
 
 void tangle_widget_set_prefer_background_size(TangleWidget* widget, gboolean prefer_background_size) {
-	if (widget->priv->prefer_background_size != prefer_background_size) {
+	if (!widget->priv->prefer_background_size_set ||
+	    widget->priv->prefer_background_size != prefer_background_size) {
 		widget->priv->prefer_background_size = prefer_background_size;
 		clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
-		g_object_notify(G_OBJECT(widget), "prefer-background_size");
+		tangle_widget_set_prefer_background_size_set(widget, TRUE);
+		g_object_notify(G_OBJECT(widget), "prefer-background-size");
 	}
 }
 
+gboolean tangle_widget_get_prefer_background_size_set(TangleWidget* widget) {
+
+	return widget->priv->prefer_background_size_set;
+}
+
+void tangle_widget_set_prefer_background_size_set(TangleWidget* widget, gboolean is_set) {
+	if (widget->priv->prefer_background_size_set != is_set) {
+		widget->priv->prefer_background_size_set = is_set;
+		clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
+		g_object_notify(G_OBJECT(widget), "prefer-background-size-set");
+	}
+}
 
 static void paint_actor_if_mapped(ClutterActor* actor) {
 	if (CLUTTER_ACTOR_IS_MAPPED(actor)) {
@@ -317,12 +333,19 @@ static void tangle_widget_paint(ClutterActor* actor) {
 	}
 }
 
+static void paint_actor_if_mapped_and_not_hiding(ClutterActor* actor) {
+	if (CLUTTER_ACTOR_IS_MAPPED(actor) &&
+	    (!TANGLE_IS_ACTOR(actor) || !TANGLE_ACTOR_IS_HIDING(TANGLE_ACTOR(actor)))) {
+		clutter_actor_paint(actor);
+	}
+}
+
 static void tangle_widget_pick(ClutterActor* actor, const ClutterColor *color) {
 	GList* child_in_list;
 
 	CLUTTER_ACTOR_CLASS(tangle_widget_parent_class)->pick(actor, color);
 
-	g_list_foreach(TANGLE_WIDGET(actor)->priv->children,(GFunc)paint_actor_if_mapped, NULL);
+	g_list_foreach(TANGLE_WIDGET(actor)->priv->children,(GFunc)paint_actor_if_mapped_and_not_hiding, NULL);
 }
 
 static void tangle_widget_get_preferred_width(TangleActor* actor, gfloat for_height, gboolean interacting, gfloat* min_width_p, gfloat* natural_width_p, gfloat* max_width_p) {
@@ -335,7 +358,9 @@ static void tangle_widget_get_preferred_width(TangleActor* actor, gfloat for_hei
 
 	widget = TANGLE_WIDGET(actor);
 
-	if (widget->priv->prefer_background_size && widget->priv->background_actor) {
+	if (widget->priv->background_actor &&
+	    ((widget->priv->prefer_background_size_set && widget->priv->prefer_background_size) ||
+	     (!widget->priv->prefer_background_size_set && !widget->priv->children))) {
 		if (TANGLE_IS_ACTOR(widget->priv->background_actor)) {
 			tangle_actor_get_preferred_width(TANGLE_ACTOR(widget->priv->background_actor), for_height, interacting, min_width_p, natural_width_p, max_width_p);
 		} else {
@@ -397,7 +422,9 @@ static void tangle_widget_get_preferred_height(TangleActor* actor, gfloat for_wi
 	
 	widget = TANGLE_WIDGET(actor);
 
-	if (widget->priv->prefer_background_size && widget->priv->background_actor) {
+	if (widget->priv->background_actor &&
+	    ((widget->priv->prefer_background_size_set && widget->priv->prefer_background_size) ||
+	     (!widget->priv->prefer_background_size_set && !widget->priv->children))) {
 		if (TANGLE_IS_ACTOR(widget->priv->background_actor)) {
 			tangle_actor_get_preferred_height(TANGLE_ACTOR(widget->priv->background_actor), for_width, interacting, min_height_p, natural_height_p, max_height_p);
 		} else {
@@ -529,6 +556,9 @@ static void tangle_widget_set_property(GObject* object, guint prop_id, const GVa
 		case PROP_PREFER_BACKGROUND_SIZE:
 			tangle_widget_set_prefer_background_size(widget, g_value_get_boolean(value));
 			break;
+		case PROP_PREFER_BACKGROUND_SIZE_SET:
+			tangle_widget_set_prefer_background_size_set(widget, g_value_get_boolean(value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -556,6 +586,9 @@ static void tangle_widget_get_property(GObject* object, guint prop_id, GValue* v
 		case PROP_PREFER_BACKGROUND_SIZE:
 			g_value_set_boolean(value, widget->priv->prefer_background_size);
 			break;
+		case PROP_PREFER_BACKGROUND_SIZE_SET:
+			g_value_set_boolean(value, widget->priv->prefer_background_size_set);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		        break;
@@ -643,6 +676,12 @@ static void tangle_widget_class_init(TangleWidgetClass* widget_class) {
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleWidget:prefer-background-size:
+	 *
+	 * Overrides the usual size request that is based on the layout and the children of the widget.
+	 * When TRUE (and #TangleWidget:prefer-background-size-set is TRUE), the size of the background actor
+	 * is returned as a size request of the widget.
+	 * 
+	 * Writing this property sets the #TangleWidget:prefer-background-size-set property as a side effect.
 	 */
 	g_object_class_install_property(gobject_class, PROP_PREFER_BACKGROUND_SIZE,
 	                                g_param_spec_boolean("prefer-background-size",
@@ -650,6 +689,22 @@ static void tangle_widget_class_init(TangleWidgetClass* widget_class) {
 	                                "Whether return the background size or the preferred size of children as a size request",
 	                                FALSE,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleWidget:prefer-background-size-set:
+	 *
+	 * Forces the widget to obey the value of the #TangleWidget:prefer-background-size property
+	 * when returning a size request. When TRUE, the size request policy depends solely on the
+	 * #TangleWidget:prefer-background-size property. When FALSE, the size request is
+	 * normally based on the layout and the size requests of the children (like #TangleWidget:prefer-background-size
+	 * was FALSE), but if the widget does not have any child, the size request of the background actor is used
+	 * instead (like #TangleWidget:prefer-background-size was TRUE).
+	 */
+	g_object_class_install_property(gobject_class, PROP_PREFER_BACKGROUND_SIZE_SET,
+	                                g_param_spec_boolean("prefer-background-size-set",
+	                                "Prefer background size",
+	                                "Whether to use the prefer-background-size property",
+	                                FALSE,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private(gobject_class, sizeof(TangleWidgetPrivate));
 }
