@@ -22,6 +22,20 @@ static gboolean on_button_clicked(TangleButton* button, gpointer user_data) {
 	tangle_actor_show(show_actor);
 }
 
+static void on_transition_completed(TangleActor* actor, ClutterActorBox* old_allocation, ClutterActorBox* current_allocation, gpointer user_data) {
+	g_print("Transition completed: %p: ", actor);
+	if (old_allocation) {
+		g_print("(%f, %f)(%f, %f) => ", old_allocation->x1, old_allocation->y1, old_allocation->x2, old_allocation->y2);
+	} else {
+		g_print("hidden => ");
+	}
+	if (current_allocation) {
+		g_print("(%f, %f)(%f, %f)\n", current_allocation->x1, current_allocation->y1, current_allocation->x2, current_allocation->y2);
+	} else {
+		g_print("hidden\n");
+	}
+}
+
 int main(int argc, char** argv) {
 	ClutterActor* stage;
 	ClutterActor* stack;
@@ -41,6 +55,7 @@ int main(int argc, char** argv) {
 
 	for (i = 0; i < 3; i++) {
 		views[i] = tangle_view_new_with_layout(tangle_stack_layout_new());
+		g_signal_connect(views[i], "transition-completed", G_CALLBACK(on_transition_completed), NULL);
 //		clutter_actor_hide(views[i]);
 		g_object_set(views[i], "transition-duration", 1500, "transition-mode", 1, NULL);
 		clutter_container_add_actor(CLUTTER_CONTAINER(stack), views[i]);
